/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 1999 - 2005, Digium, Inc.
 *
 * Mark Spencer <markster@digium.com>
 * Russell Bryant <russelb@clemson.edu>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*! \file
 *
 * \brief Applications to test connection and produce report in text file
 *
 * \author Mark Spencer <markster@digium.com>
 * \author Russell Bryant <russelb@clemson.edu>
 * 
 * \ingroup applications
 */

#include "asterisk.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 237924 $")

#include <sys/stat.h>

#include "asterisk/paths.h"	/* use ast_config_AST_LOG_DIR */
#include "asterisk/channel.h"
#include "asterisk/module.h"
#include "asterisk/lock.h"
#include "asterisk/app.h"
#include "asterisk/pbx.h"
#include "asterisk/utils.h"
#include "asterisk/astdb.h"
#include "asterisk/linkedlists.h"

/*** DOCUMENTATION
	<application name="TestServer" language="en_US">
		<synopsis>
			Execute DATABASE Test
		</synopsis>
		<syntax />
		<description>
			<para></para>
		</description>
	</application>
	<application name="TestDB" language="en_US">
		<synopsis>
			Execute DATABASE TEST.
		</synopsis>
		<syntax>
		</syntax>
		<description>
			<para></para>
		</description>
		<see-also>
		</see-also>
	</application>
 ***/

static char *app = "TestDB";

static int testdb_exec(struct ast_channel *chan, void *data)
{
	int counter=0;
	unsigned long sum=0;
	int min=10;
	int max=0;
	int now=0;
	int dmin=10;
	int dmax=0;
	int dnow=0;
	unsigned long dsum=0;
	int longer=0;
	int dlonger=0;
	int longer2=0;
	int dlonger2=0;
	double avg=0.0;
	double davg=0.0;
	struct timeval start;
	struct timeval startall;
	char *value=NULL;

	int howmuch=10000;

	startall=ast_tvnow();
	for(counter=0;counter<howmuch;counter++)
	{
		asprintf(&value,"entry%d",counter);
		start=ast_tvnow();
		ast_db_put("TESTDB",value,value);
		now=(int)ast_tvdiff_us(ast_tvnow(),start);
		if(now>max) {
			max=now;
		}
		if(now<min) {
			min=now;
		}
		if(now>(min+10)) {
			longer++;
		}
		if(now>(min+100)) {
			longer--;
			longer2++;
		}
		sum+=now;
		start=ast_tvnow();
		ast_db_del("TESTDB",value);
		dnow=(int)ast_tvdiff_us(ast_tvnow(),start);
		if(dnow>dmax) {
			dmax=dnow;
		}
		if(dnow<dmin) {
			dmin=dnow;
		}
		if(dnow>(dmin+10)) {
			dlonger++;
		}
		if(dnow>(dmin+100)) {
			dlonger--;
			dlonger2++;
		}
		dsum+=dnow;
		//ast_verbose("Writing Entry to Astdb tooks %d usec\n",(int)ast_tvdiff_us(ast_tvnow(),start));

	}
	ast_verbose("TESTDB RESULT: Overall Time: %d ms\n",(int)ast_tvdiff_ms(ast_tvnow(),startall));
	ast_verbose("Writing %d Entries to Astdb tooks %ld usec\n",howmuch,sum);
	ast_verbose("Deleting %d Entries from Astdb tooks %ld usec\n",howmuch,dsum);
	avg=sum/howmuch;
	davg=dsum/howmuch;
	ast_verbose("WRITE: \t avg time: %f usec\t max time: %d usec\t min time: %d usec \t longer than min+10us: %d \t min+100us: %d\n",avg,max,min,longer,longer2);
	ast_verbose("DEL: \t avg time: %f usec\t max time: %d usec\t min time: %d usec \t longer than min+10us: %d \t min+100us: %d\n\n",davg,dmax,dmin,dlonger,dlonger2);
/*
	min=100;
	max=0;
	sum=0;
	now=0;
	startall=ast_tvnow();
	for(counter=0;counter<howmuch;counter++)
	{
		asprintf(&value,"entry%d",counter);
		start=ast_tvnow();
		ast_db_del("TESTDB",value);
		now=(int)ast_tvdiff_us(ast_tvnow(),start);
		if(now>max) {
			max=now;
		}
		if(now<min) {
			min=now;
		}
		sum+=now;
		//ast_verbose("Deleting Entry from Astdb tooks %d usec\n",(int)ast_tvdiff_us(ast_tvnow(),start));

	}
	avg=sum/howmuch;
	ast_verbose("Deleting %d Entries from Astdb tooks %d usec overall: %d\n",howmuch,sum,(int)ast_tvdiff_us(ast_tvnow(),startall));
	ast_verbose("avg time: %f usec\t max time: %d usec\tmin time: %d usec\n\n",avg,max,min);
	*/
	return 0;
}

static int unload_module(void)
{

	return ast_unregister_application(app);
}

static int load_module(void)
{
	return ast_register_application_xml(app, testdb_exec);
}

AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Interface DATABASE Test Application");
